@@ -36,22 +36,28 @@ def closed_eids(line):
     matches = TICKET_EID_RGX.findall(' '.join(words[idx+1:]))
     return [int(i) for i in matches]
 
-_TICKET_RQL = '''Any X
-WHERE X eid %(t)s,
-      X concerns P,
-      P source_repository R,
-      R eid %(r)s
-'''
-def tickets(session, eids, repo_eid):
-    data = {'r': repo_eid, 't': None}
-    for ticket_eid in eids:
-        data['t'] = ticket_eid
+def tickets(session, ticket_eids, repository_eid):
+    """return rset for a ticket referenced in a commit message.
+
+    The ticket must come from a project using given repository as
+    source_repository (or one of its subproject).
+
+    Return None if no match.
+    """
+    for ticket_eid in ticket_eids:
         try:
-            rset = session.execute(_TICKET_RQL, data)
+            rset = session.execute('Any X, P WHERE X eid %(x)s, X concerns P',
+                                   {'x': ticket_eid})
         except TypeResolverException:
             continue
-        for ticket in rset.entities():
-            yield ticket
+        project = rset.get_entity(0, 1)
+        while project:
+            if project.source_repository:
+                if project.source_repository[0].eid == repository_eid:
+                    yield rset.get_entity(0, 0)
+                else:
+                    break
+            project = project.subproject_of and project.subproject_of[0]
 
 # search for ticket closing instructions #######################################
 
